@@ -256,7 +256,8 @@ class BootLoader(object):
                             "noht", "acpi", "video", "pci", "nodmraid",
                             "nompath", "nomodeset", "noiswmd", "fips",
                             "selinux", "biosdevname", "ipv6.disable",
-                            "net.ifnames"]
+                            "net.ifnames", "vconsole.keymap", "locale",
+                            "rd.vconsole.keymap", "keymap"]
     preserve_args = []
 
     _trusted_boot = False
@@ -712,7 +713,7 @@ class BootLoader(object):
         if not self._is_valid_size(device, desc=_(self.stage2_description)):
             valid = False
 
-        if not self._is_valid_location(device,
+        if self.stage2_max_end and not self._is_valid_location(device,
                                        max_end=self.stage2_max_end,
                                        desc=_(self.stage2_description)):
             valid = False
@@ -805,7 +806,8 @@ class BootLoader(object):
         #
         # FIPS
         #
-        if flags.cmdline.get("fips") == "1":
+        boot_device = storage.mountpoints.get("/boot")
+        if flags.cmdline.get("fips") == "1" and boot_device:
             self.boot_args.add("boot=%s" % self.stage2_device.fstabSpec)
 
         #
@@ -1576,8 +1578,14 @@ class GRUB2(GRUB):
 
         # make sure the default entry is the OS we are installing
         if self.default is not None:
-            entry_title = "0"
-            rc = iutil.execInSysroot("grub2-set-default", [entry_title])
+            # find the index of the default image
+            try:
+                default_index = self.images.index(self.default)
+            except ValueError:
+                log.warn("Failed to find default image (%s), defaulting to 0", self.default.label)
+                default_index = 0
+
+            rc = iutil.execInSysroot("grub2-set-default", [str(default_index)])
             if rc:
                 log.error("failed to set default menu entry to %s", productName)
 
