@@ -119,7 +119,8 @@ def _get_property(object_path, prop, interface_name_suffix=""):
     try:
         prop = proxy.Get('(ss)', interface_name, prop)
     except GLib.GError as e:
-        if "org.freedesktop.DBus.Error.AccessDenied" in e.message:
+        if ("org.freedesktop.DBus.Error.AccessDenied" in e.message or
+            "org.freedesktop.DBus.Error.InvalidArgs" in e.message):
             return None
         elif "org.freedesktop.DBus.Error.UnknownMethod" in e.message:
             raise UnknownMethodGetError
@@ -382,6 +383,28 @@ def nm_device_perm_hwaddress(name):
     """
     return nm_device_property(name, "PermHwAddress")
 
+def nm_device_valid_hwaddress(name):
+    """Return valid hardware address of device depending on type of the device
+       ('PermHwAddress' property for wired and wireless or 'HwAddress' property for others)
+       :param name: name of device
+       :type name: str
+       :return: active hardware address of device
+                ('HwAddress' or 'PermHwAddress' property)
+       :rtype: str
+       :raise UnknownDeviceError: if device is not found
+       :raise PropertyNotFoundError: if property is not found
+    """
+    if nm_device_type_is_ethernet(name) or nm_device_type_is_wifi(name):
+        try:
+            return nm_device_perm_hwaddress(name)
+        except PropertyNotFoundError:
+            # TODO: Remove this if everything will work well
+            # fallback solution
+            log.warning("Device %s don't have property PermHwAddress", name)
+            return nm_device_hwaddress(name)
+    else:
+        return nm_device_hwaddress(name)
+
 def nm_device_active_con_uuid(name):
     """Return uuid of device's active connection
 
@@ -562,7 +585,7 @@ def nm_hwaddr_to_device_name(hwaddr):
         :rtype: str
     """
     for device in nm_devices():
-        if nm_device_perm_hwaddress(device).upper() == hwaddr.upper():
+        if nm_device_valid_hwaddress(device).upper() == hwaddr.upper():
             return device
     return None
 
@@ -614,7 +637,7 @@ def _device_settings(name):
         settings = _find_settings(name, 'connection', 'interface-name')
         if not settings:
             try:
-                hwaddr_str = nm_device_perm_hwaddress(name)
+                hwaddr_str = nm_device_valid_hwaddress(name)
             except PropertyNotFoundError:
                 settings = []
             else:
@@ -723,7 +746,8 @@ def nm_device_setting_value(name, key1, key2):
     """
     settings_paths = _device_settings(name)
     if not settings_paths:
-        raise SettingsNotFoundError(name)
+        value = None
+        return value
     else:
         settings_path = settings_paths[0]
     proxy = _get_proxy(object_path=settings_path, interface_name="org.freedesktop.NetworkManager.Settings.Connection")
