@@ -655,6 +655,7 @@ blacklist nouveau
             "dev-libs/satyr",
             "dev-python/python-blivet",
             "dev-python/python-meh",
+            "dev-util/glade",
             "dev-util/pykickstart",
             "libselinux",
             "sys-process/audit",
@@ -669,6 +670,9 @@ blacklist nouveau
         try:
             repo = self._backend.entropy.installed_repository()
 
+            if not self.is_virtualbox():
+                self.remove_package("virtualbox-guest-additions")
+
             for package in packages:
 
                 pkg_id, _pkg_rc = repo.atomMatch(package)
