@@ -507,9 +507,9 @@ class GraphicalUserInterface(UserInterface):
         ANACONDA_WINDOW_GROUP.add_window(self.mainWindow)
 
     basemask = "pyanaconda.ui"
-    basepath = os.path.dirname(__file__)
+    basepath = os.path.normpath(os.path.dirname(__file__))
     updatepath = "/tmp/updates/pyanaconda/ui"
-    sitepackages = [os.path.join(dir, "pyanaconda", "ui")
+    sitepackages = [os.path.normpath(os.path.join(dir, "pyanaconda", "ui"))
                     for dir in site.getsitepackages()]
     pathlist = set([updatepath, basepath] + sitepackages)
 
